//
// Copyright (c) ZeroC, Inc. All rights reserved.
//

// No pragma once as this file can be included several times in a translation
// unit

#if defined(_MSC_VER)
#   pragma warning(pop)
#elif defined(__clang__)
#   pragma clang diagnostic pop
#elif defined(__GNUC__)
#   pragma GCC diagnostic pop
#endif
