# -*- coding: utf-8 -*-
#
# Copyright (c) ZeroC, Inc. All rights reserved.
#
#
# Ice version 3.7.8
#
# <auto-generated>
#
# Generated from file `SliceChecksumDict.ice'
#
# Warning: do not edit this file.
#
# </auto-generated>
#

from sys import version_info as _version_info_
import Ice, IcePy

# Start of module Ice
_M_Ice = Ice.openModule('Ice')
__name__ = 'Ice'

if '_t_SliceChecksumDict' not in _M_Ice.__dict__:
    _M_Ice._t_SliceChecksumDict = IcePy.defineDictionary('::Ice::SliceChecksumDict', (), IcePy._t_string, IcePy._t_string)

# End of module Ice

Ice.sliceChecksums["::Ice::SliceChecksumDict"] = "d733c49f2c6d79a346ca7d695f0519"
