//
// Copyright (c) ZeroC, Inc. All rights reserved.
//

#ifndef ICE_UUID_H
#define ICE_UUID_H

#include <Ice/Config.h>
#include <IceUtil/UUID.h>

namespace Ice
{

/** Generates a UUID. */
using IceUtil::generateUUID;

}

#endif
