# -*- coding: utf-8 -*-
#
# Copyright (c) ZeroC, Inc. All rights reserved.
#
#
# Ice version 3.7.8
#
# <auto-generated>
#
# Generated from file `RouterF.ice'
#
# Warning: do not edit this file.
#
# </auto-generated>
#

from sys import version_info as _version_info_
import Ice, IcePy

# Start of module Glacier2
_M_Glacier2 = Ice.openModule('Glacier2')
__name__ = 'Glacier2'

if 'Router' not in _M_Glacier2.__dict__:
    _M_Glacier2._t_RouterDisp = IcePy.declareClass('::Glacier2::Router')
    _M_Glacier2._t_RouterPrx = IcePy.declareProxy('::Glacier2::Router')

# End of module Glacier2
